@@ -16,37 +16,44 @@
 #include <sharing-plugin-interface.h>
 #include <sharing-entry.h>
 #include <sharing-tag.h>
-#ifdef WITH_ALBUM_SUPPORT
-#include <sharing-service-option.h>
-#endif /* WITH_ALBUM_SUPPORT */
-
 
 #include "picasa_post.h"
 #include "picasa_http.h"
 #include "picasa_xml.h"
 
-
-/** Globals **/
-
-/* TODO: get rid of globals */
-
 /**
-   g_response_buffer
+ * g_response_buffer:
+ *
+ * Buffer where CURL response is stored.
 */
 GString *g_response_buffer = NULL;
 /**
-   g_auth
+ * g_auth:
+ *
+ * Global variable for login auth string.
 */
 gchar *g_auth = NULL;
 /**
-   g_photo_id
+ * g_photo_url:
+ *
+ * Url where the photo is uploaded.
 */
 gchar *g_photo_url = NULL;
 
 
 
 /**
-   picasa_post_progress_data_t
+ * picasa_post_progress_data_t:
+ * @dead_mans_switch: -
+ * @cont: Continue flag if user cancels
+ * @transfer: Sharing transfer
+ * @entry: Sharing entry
+ * @account: Sharing account
+ * @request: HTTP request
+ * @total_size: total size to upload
+ * @uploaded_size: uploaded size
+ *
+ * Data structure for CURL progress callback.
 */
 typedef struct {
   gboolean *dead_mans_switch;
@@ -60,7 +67,9 @@ typedef struct {
 } picasa_post_progress_data_t;
 
 /**
-   picasa_post_response_type_t
+ * picasa_post_response_type_t:
+ *
+ * Response types based on made request.
 */
 typedef enum {
   RES_LOGIN = 0x01,
@@ -71,17 +80,11 @@ typedef enum {
 } picasa_post_response_type_t;
   
 /**
-   picasa_post_tag_type_t
-*/
-typedef enum {
-  NORMAL_TAG = 0x01,
-  GPS_TAG = 0x02,
-  GEO_TAG = 0x03,
-  FAV_TAG = 0x04
-} picasa_post_tag_type_t;
-
-/**
-   picasa_post_response_t
+ * picasa_post_response_t:
+ * @code: HTTP response code
+ * @msg: Error message or other information
+ *
+ * HTTP post response structure.
 */
 typedef struct {
   gint code;
@@ -95,7 +98,9 @@ typedef struct {
 
 
 /**
-   picasa_post_init_response_buffer
+ * picasa_post_init_response_buffer:
+ *
+ * Creates new string buffer for responses.
 */
 static void
 picasa_post_init_response_buffer () {
@@ -103,10 +108,10 @@ picasa_post_init_response_buffer () {
   g_response_buffer = g_string_new (NULL);
 }
 
-
-
 /**
-   picasa_post_clear_response_buffer
+ * picasa_post_clear_response_buffer:
+ *
+ * Frees response buffer.
 */
 static void
 picasa_post_clear_response_buffer () {
@@ -116,7 +121,9 @@ picasa_post_clear_response_buffer () {
 }
 
 /**
-  picasa_post_free_globals
+ * picasa_post_free_globals:
+ *
+ * Frees all global variables.
 */
 static void
 picasa_post_free_globals () {
@@ -137,7 +144,12 @@ picasa_post_free_globals () {
 }
 
 /**
-   picasa_post_convert_http_error
+ * picasa_post_convert_http_error:
+ * @state: HTTP request state
+ *
+ * Converts request state for error code.
+ *
+ * Returns: Error code
 */
 static picasa_response_codes_t
 picasa_post_convert_http_error (http_request_state_t state) {
@@ -160,9 +172,13 @@ picasa_post_convert_http_error (http_request_state_t state) {
   return status;
 }
 
-#ifdef WITH_ALBUM_SUPPORT
 /**
-   picasa_post_convert_update_options
+ * picasa_post_convert_update_options:
+ * @status: Update options status
+ *
+ * Converts request status to result code.
+ *
+ * Returns: Result code
 */
 static SharingPluginInterfaceUpdateOptionsResult
 picasa_post_convert_update_options (gint status) {
@@ -187,11 +203,14 @@ picasa_post_convert_update_options (gint status) {
 
   return result;
 }
-#endif /* WITH_ALBUM_SUPPORT */
-
 
 /**
-   picasa_post_convert_validate_response
+ * picasa_post_convert_validate_response:
+ * @res: HTTP post response
+ *
+ * Converts validate response to result code.
+ *
+ * Returns: Result code
 */
 static SharingPluginInterfaceAccountValidateResult
 picasa_post_convert_validate_response (picasa_post_response_t res) {
@@ -230,9 +249,13 @@ picasa_post_convert_validate_response (picasa_post_response_t res) {
   return result;
 }
 
-
 /**
-   picasa_post_convert_send_response
+ * picasa_post_convert_send_response:
+ * @res: HTTP post response 
+ *
+ * Converts send response to result code
+ *
+ * Returns: Result code
 */
 static SharingPluginInterfaceSendResult
 picasa_post_convert_send_response (picasa_post_response_t res) {
@@ -290,7 +313,13 @@ picasa_post_convert_send_response (picasa_post_response_t res) {
 }
 
 /**
-  picasa_post_parse_response
+ * picasa_post_parse_response:
+ * @data: Progress data
+ * @type: Response type
+ *
+ * Parses HTTP response data.
+ *
+ * Returns: Response struct with code and message
 */
 static picasa_post_response_t
 picasa_post_parse_response (picasa_post_progress_data_t *data,
@@ -309,10 +338,8 @@ picasa_post_parse_response (picasa_post_progress_data_t *data,
   gchar *tmp = (gchar*)g_response_buffer->str;
   gint count = 0;
   guint offset = 0;
-#ifdef WITH_ALBUM_SUPPORT
   GSList *albums = NULL;
   gboolean success = FALSE;
-#endif /* WITH_ALBUM_SUPPORT */
   picasa_post_response_t res = { 0, NULL };
 
   tokens = g_strsplit_set ((gchar*)g_response_buffer->str,
@@ -345,13 +372,6 @@ picasa_post_parse_response (picasa_post_progress_data_t *data,
 	ULOG_DEBUG_L ("%s: uploaded photo id parsed (%s)", 
 		      fn, g_photo_url);
       }
-#ifdef WITH_ALBUM_SUPPORT
-      else if (type == RES_GET_ALBUMS) {
-	albums = picasa_xml_parse_album_response (iter);
-	ULOG_DEBUG_L ("%s: albums parsed (%0X)", 
-		      fn, (guint)albums);
-      }
-#endif /* WITH_ALBUM_SUPPORT */
       else if (type == RES_GET_COORDINATES) {
 	res.msg = picasa_xml_parse_gps_response ((tmp+(offset+count)));
 	ULOG_DEBUG_L ("%s: coordinates parsed = %s", 
@@ -374,24 +394,6 @@ picasa_post_parse_response (picasa_post_progress_data_t *data,
 
   }
   
-#ifdef WITH_ALBUM_SUPPORT
-  if (albums != NULL && res.code == HTTP_OK) {
-    ULOG_DEBUG_L ("%s: setting up account option values (%d) to account %s", 
-		  fn, g_slist_length (albums), 
-		  sharing_account_get_param (data->account, "username"));
-    
-    success = sharing_account_set_option_values (data->account,
-						 "albums",
-						 albums);
-    if (success == FALSE) {
-      ULOG_DEBUG_L ("%s: failed to update albums",fn);
-      res.code = GENERAL_ERROR;
-    }
-
-    sharing_service_option_values_free (albums);
-    
-  }
-#endif /* WITH_ALBUM_SUPPORT */
   
   g_strfreev (tokens);
 
@@ -401,7 +403,16 @@ picasa_post_parse_response (picasa_post_progress_data_t *data,
 }
 
 /**
-  picasa_post_progress
+ * picasa_post_progress:
+ * @clientp: Pointer to client data
+ * @dltotal: Total DL size
+ * @dlnow: DL size now
+ * @ultotal: Total UL size
+ * @ulnow: UL size now
+ *
+ * Progress function which CURL calls once a sec.
+ *
+ * Returns: 0 for continue and 1 if CURL needs to exit
 */
 static int
 picasa_post_progress (void *clientp,
@@ -444,27 +455,34 @@ picasa_post_progress (void *clientp,
     return 1;
   }
 
-  /* if all fine update progress */
-  progress = (data->uploaded_size + ulnow) / data->total_size;
-  if (progress > 1.0) {
-    ULOG_DEBUG_L ("%s: FULL PROGRESS", fn);
-    progress = 1.0;
+  /* if total_size defined. Not defined for example account validation */
+  if (data->total_size > 0) {
+    progress = (data->uploaded_size + ulnow) / data->total_size;
+    if (progress > 1.0) {
+      ULOG_DEBUG_L ("%s: FULL PROGRESS", fn);
+      progress = 1.0;
+    }
+
+    ULOG_DEBUG_L ("%s: progress = %f", fn, progress);
+    
+    sharing_transfer_set_progress (data->transfer, progress);
   }
 
-  ULOG_DEBUG_L ("%s: progress = %f", fn,progress);
-
-  sharing_transfer_set_progress (data->transfer, progress);
-
   /* pull dead_mans_switch */
   *(data->dead_mans_switch) = FALSE;
-
+  
   return 0;
-
 }
 
 
 /**
-   picasa_post_login
+ * picasa_post_login:
+ * @con: Conic connection
+ * @daya: Progress data
+ *
+ * Creates and sends login request to service.
+ *
+ * Returns: Request response
 */
 static picasa_post_response_t
 picasa_post_login (ConIcConnection *con,
@@ -490,7 +508,7 @@ picasa_post_login (ConIcConnection *con,
 
   data->request = http;
 
-  picasa_http_add_post_data (http, "accountType", "HOSTED_OR_GOOGLE");
+  picasa_http_add_post_data (http, "accountType", "GOOGLE");
   picasa_http_add_post_data (http, "Email", uname);
   picasa_http_add_post_data (http, "Passwd", passwd);
   picasa_http_add_post_data (http, "service", SERVICE_C);
@@ -503,6 +521,7 @@ picasa_post_login (ConIcConnection *con,
 				   g_response_buffer);
   
   picasa_http_set_connection (http, con);
+  picasa_http_set_progress_cb (http, picasa_post_progress, data);
 
   /* send http post */
   ULOG_DEBUG_L ("%s: Sending post to %s",fn,LOGIN_URL_C);
@@ -534,9 +553,14 @@ picasa_post_login (ConIcConnection *con,
 }
 
 
-#ifdef WITH_ALBUM_SUPPORT
 /**
-  picasa_post_get_albums
+ * picasa_post_get_albums:
+ * @con: Conic connection
+ * @data: Progress data
+ *
+ * Creates album update request and sends to service.
+ *
+ * Returns: Request response
 */
 static picasa_post_response_t 
 picasa_post_get_albums (ConIcConnection *con,
@@ -570,7 +594,6 @@ picasa_post_get_albums (ConIcConnection *con,
 				   g_response_buffer);
 
   picasa_http_set_connection (http, con);
-  
   picasa_http_set_progress_cb (http, picasa_post_progress, data);
 
   /* send http post */
@@ -578,7 +601,7 @@ picasa_post_get_albums (ConIcConnection *con,
 
   *data->dead_mans_switch = FALSE;
 
-  status = picasa_http_send (session, http, GET);
+  status = picasa_http_send (http, GET);
 
   *data->dead_mans_switch = FALSE;
 
@@ -598,10 +621,16 @@ picasa_post_get_albums (ConIcConnection *con,
   
   return res;
 }
-#endif /* WITH_ALBUM_SUPPORT */
 
 /**
-  picasa_post_upload_item
+ * picasa_post_upload_item:
+ * @media: Media to upload
+ * @con: Conic connection
+ * @data: Progress data
+ *
+ * Creates item upload request and send it to service.
+ *
+ * Returns: Request response
 */
 static picasa_post_response_t
 picasa_post_upload_item (SharingEntryMedia *media,
@@ -620,11 +649,7 @@ picasa_post_upload_item (SharingEntryMedia *media,
   gchar *title = sharing_entry_media_get_title (media);
   gchar *desc = (gchar*)sharing_entry_media_get_desc (media);
 
-#ifdef WITH_ALBUM_SUPPORT
-  gchar *album_id = (gchar*)sharing_entry_get_option (data->entry, "albums");
-#else
   gchar *album_id = "default";
-#endif /* WITH_ALBUM_SUPPORT */
   
   gchar *url = g_strconcat (FEED_URL_C, "/albumid/", album_id, NULL);
   gchar *auth = g_strconcat ("GoogleLogin auth=", g_auth, NULL);
@@ -706,7 +731,15 @@ picasa_post_upload_item (SharingEntryMedia *media,
 
 
 /**
-  picasa_post_update_media
+ * picasa_post_update_media:
+ * @tags: Tags
+ * @gps: GPS location
+ * @con: Conic connection
+ * @data: Progress data
+ *
+ * Creates media update reqeust and sends it to service.
+ *
+ * Returns: Request response
 */
 static picasa_post_response_t
 picasa_post_update_media (gchar *tags,
@@ -790,7 +823,16 @@ picasa_post_update_media (gchar *tags,
 }
 
 /**
-  picasa_post_get_gps_coordinates
+ * picasa_post_get_gps_coordinates:
+ * @con: Conic connection
+ * @data: Progress data
+ * @country: Country
+ * @city: City
+ * @suburb: Suburb aka district
+ *
+ * Creates geocoding request and sends it to service.
+ *
+ * Return: Request response.
 */
 static picasa_post_response_t 
 picasa_post_get_gps_coordinates (ConIcConnection *con,
@@ -869,105 +911,15 @@ picasa_post_get_gps_coordinates (ConIcConnection *con,
   return res;
 }
 
-#ifdef WITH_ALBUM_SUPPORT
 /**
-  picasa_post_add_tag
-*/
-static picasa_post_response_t
-picasa_post_add_tag (gchar *tag,
-		     gint type,
-		     ConIcConnection *con,
-		     picasa_post_progress_data_t *data) {
-  
-#ifdef DEBUG
-  gchar *fn = "picasa_post_add_tag()";
-#endif
-
-  gint status = 0;
-  picasa_post_response_t res = { 0, NULL };
-  gchar *tag_entry = NULL;
-  gchar *length = NULL;
-
-  gchar *auth = g_strconcat ("GoogleLogin auth=", g_auth, NULL);
-
-  if (g_photo_url == NULL) {
-    ULOG_WARN_L ("%s: photo url NULL -> tags will not be updated", fn);
-    res.code = GENERAL_ERROR;
-    return res;
-  }
-
-  ULOG_DEBUG_L ("%s: uploading to url %s", fn, g_photo_url);
-
-  http_request_t *http = picasa_http_new (g_photo_url);
-
-  if (!http) {
-    ULOG_ERR_L ("%s: failed to allocate new http request, aborting upload", fn);
-    res.code = GENERAL_ERROR;
-    return res;
-  }
-
-  data->request = http;
-
-  picasa_http_add_header_data (http, "GData-Version", "2");
-  picasa_http_add_header_data (http, "Authorization", auth);
-  picasa_http_add_header_data (http, "Content-Type", "application/atom+xml");
-  picasa_http_add_header_data (http, "Expect", "Expect: 100-continue");
-
-  if (type == NORMAL_TAG) {
-    tag_entry = picasa_xml_create_tag_entry (tag);
-  }
-  else if (type == GPS_TAG) {
-    tag_entry = picasa_xml_create_gpstag_entry (tag);
-  } 
-  else {
-    ULOG_DEBUG_L ("%s: unknown type", fn);
-    picasa_http_free (http);
-    res.code = GENERAL_ERROR;
-    return res;
-  }
-
-  picasa_http_append_prefix (http, tag_entry);
-  g_free (tag_entry);
-
-  length = g_strdup_printf("%d", http->prefix.data->len);
-  picasa_http_add_header_data (http, "Content-Length", length);
-  g_free(length);
-
-  picasa_post_init_response_buffer ();
-  picasa_http_set_response_buffer (http,
-				   PICASA_POST_BUF_LIMIT_C,
-				   g_response_buffer);
-  
-  picasa_http_set_connection (http, con);
-
-  ULOG_DEBUG_L ("%s: Sending post to %s", fn, g_photo_url);
-
-  *data->dead_mans_switch = FALSE;
-
-  status = picasa_http_send (http, POST);
-
-  *data->dead_mans_switch = FALSE;
-
-  ULOG_DEBUG_L ("%s: sharing_http_request_send returned %d",fn,status);
-
-  /* send successfully, check picasa response */
-  if (status == 0) {
-    res = picasa_post_parse_response (data, RES_UPDATE);
-  } else {
-    ULOG_WARN_L ("%s: failed to update tag %s", fn, tag);
-  }
-
-  picasa_http_free (http);
-  picasa_post_clear_response_buffer ();
-  g_free (auth);
-
-  return res;
-}
-#endif /* WITH_ALBUM_SUPPORT */
-
-
-/**
-   picasa_post_add_tags
+ * picasa_post_add_tags:
+ * @media: Sharing media
+ * @con: Conic connection
+ * @data: Progress data
+ *
+ * Checks tags and geotags from media and updates to service.
+ *
+ * Returns: Response code
 */
 static gint
 picasa_post_add_tags (SharingEntryMedia *media,
@@ -1004,6 +956,7 @@ picasa_post_add_tags (SharingEntryMedia *media,
 	if(first == TRUE) {
 	  tag_str = g_string_new (sharing_tag_get_word (tag));
 	  first = FALSE;
+	  tags = tag_str->str;
 	} else {
 	  g_string_append (tag_str, ",");
 	  g_string_append (tag_str, sharing_tag_get_word (tag));
@@ -1086,62 +1039,6 @@ picasa_post_add_tags (SharingEntryMedia *media,
 }
 
 
-#ifdef WITH_ALBUM_SUPPORT
-/**
-   picasa_post_update_albums
-*/
-SharingPluginInterfaceUpdateOptionsResult
-picasa_post_update_albums (SharingAccount *account,
-			   ConIcConnection *con,
-			   gboolean *cont,
-			   gboolean *dead_mans_switch) {
-
-#ifdef DEBUG
-  gchar *fn = "picasa_post_update_albums()";
-#endif
-
-  ULOG_DEBUG_L ("Entered %s",fn);
-
-  SharingPluginInterfaceUpdateOptionsResult
-    status = 0;
-  picasa_post_response_t res = { 0, NULL };
-
-  picasa_post_progress_data_t data;
-
-  *dead_mans_switch = FALSE;
-
-  /* fill data structure */
-  data.dead_mans_switch = dead_mans_switch;
-  data.cont = cont;
-  data.account = account;
-  data.transfer = NULL;
-  data.entry = NULL;
-  data.request = NULL;
-
-  http_session_t session = picasa_http_init ();
-
-  ULOG_DEBUG_L ("%s: LOGIN",fn);  
-
-  /* login to validate */
-  res = picasa_post_login (session, con, &data);
-
-  /* fetch albums for updating them */
-  ULOG_DEBUG_L ("%s: GET ALBUMS",fn);
-
-  res = picasa_post_get_albums (session, con, &data);
-
-  status = picasa_post_convert_update_options (res.code);
-
-  picasa_http_uninit (session);
-  picasa_post_free_globals ();
-  if (res.msg != NULL) {
-    g_free (res.msg);
-    res.msg = NULL;
-  }
-
-  return status;
-}
-#endif /* WITH_ALBUM_SUPPORT */
 
 
 /*
@@ -1149,53 +1046,16 @@ picasa_post_update_albums (SharingAccount *account,
 */
 
 
-#ifdef WITH_ALBUM_SUPPORT
 /**
-   picasa_post_update_options
-*/
-gpointer
-picasa_post_update_options (gpointer data) {
-
-#ifdef DEBUG
-  gchar *fn = "picasa_post_update_options()";
-#endif
-
-  picasa_options_data_t *opt_data = (picasa_options_data_t*)data;
-  SharingPluginInterfaceUpdateOptionsResult status = 
-    SHARING_UPDATE_OPTIONS_ERROR_UNKNOWN;
-
-  ULOG_DEBUG_L ("%s: entered",fn);
-  
-  status = picasa_post_update_albums (opt_data->account,
-				      opt_data->con,
-				      opt_data->cont,
-				      opt_data->dead_mans_switch);
-  
-  ULOG_DEBUG_L ("%s: updated options with status %d",fn,status);
- 
-  if (opt_data->cb_func != NULL) {
-    /* callback */
-    void (*fp) (SharingPluginInterfaceUpdateOptionsResult, gpointer);
-    /* UpdateOptionsCallback*/ 
-    fp = opt_data->cb_func;
-    fp (status, opt_data->cb_data);
-  } else {
-    ULOG_CRIT_L ("%s: callback pointer NULL", fn);
-  }
-
-  /* free data */
-  /* g_free (opt_data); */
-
-  opt_data->active = FALSE;
-
-  ULOG_DEBUG_L ("%s: update_options DONE", fn);
-
-  return NULL;  
-}
-#endif /* WITH_ALBUM_SUPPORT */
-
-/**
-   picasa_post_validate_account
+ * picasa_post_validate_account:
+ * @account: Account to validate
+ * @con: Conic connection
+ * @cont: Continue flag is user cancels
+ * @dead_mans_switch: -
+ *
+ * Tries to validate account by logging in to service.
+ *
+ * Returns: Validation result
 */
 SharingPluginInterfaceAccountValidateResult
 picasa_post_validate_account (SharingAccount *account,
@@ -1213,6 +1073,7 @@ picasa_post_validate_account (SharingAccount *account,
     status = SHARING_ACCOUNT_VALIDATE_FAILED;
 
   picasa_post_response_t res = { 0, NULL };
+
   picasa_post_progress_data_t data;
 
   *dead_mans_switch = FALSE;
@@ -1225,6 +1086,8 @@ picasa_post_validate_account (SharingAccount *account,
   data.transfer = NULL;
   data.entry = NULL;
   data.request = NULL;
+  data.total_size = 0;
+  data.uploaded_size = 0;
 
   ULOG_DEBUG_L ("%s: LOGIN",fn);  
 
@@ -1240,7 +1103,14 @@ picasa_post_validate_account (SharingAccount *account,
 
 
 /**
-   picasa_post_send_to_service
+ * picasa_post_send_to_service:
+ * @transfer: Sharing transfer data
+ * @con: Conic connection
+ * @dead_mans_switch: -
+ * 
+ * Uploads all media under one transfer into service.
+ *
+ * Returns: Send result
 */
 SharingPluginInterfaceSendResult 
 picasa_post_upload_to_service (SharingTransfer *transfer,
@@ -1307,7 +1177,12 @@ picasa_post_upload_to_service (SharingTransfer *transfer,
 	status = picasa_post_convert_send_response (res);
 	break;
       }
+    } else {
+      /* item already sent previously */
+      /* only update uploaded size for progress bar */
+      data.uploaded_size += sharing_entry_media_get_size (media);
     }
+
   }
 
   picasa_post_free_globals ();
